import org.apache.commons.math3.ode.*;

/** Example of a system of first order differential equations (ODEs): Transient response of a series RLC circuit.<br/><br/>
  * <code>i = i<sub>R</sub> = i<sub>L</sub> = i<sub>C</sub> ; &nbsp; du<sub>C</sub>/dt = i<sub>C</sub>/C ; &nbsp; di<sub>L</sub>/dt = u<sub>L</sub>/L ;
  * &nbsp; u<sub>R</sub> = R&middot;i<sub>R</sub> ; &nbsp; u<sub>L</sub> = cos 2 &pi; f t - u<sub>C</sub> - u<sub>R</sub> ;
  * </code><br>
  * <br>
  * See <a target="_blank" href="../CommonsMath_ODE_RLC_circuit~Demo.jxn">CommonsMath_ODE_RLC_circuit~Demo.jxn</a>.
  */
public class CommonsMath_ODE_RLC_circuit implements FirstOrderDifferentialEquations
{
// TODO: define global parameters
   double R, L, C, omega;

// TODO: adapt class name and input parameters
   /** Stores the parameters of the RLC circuit and the frequency of the sinusodial voltage source.
     * @param R resistance
     * @param L inductance
     * @param C capacitance
     * @param lgf log10 of the frequency
     */
   public CommonsMath_ODE_RLC_circuit( double R, double L, double C, double lgf )
   {
      this.R = R;
      this.L = L;
      this.C = C;
      omega = Math.pow( 10., lgf );
   }
   
// --- interface FirstOrderDifferentialEquations ---

   /** Returns the number of states(=2). */
   public int getDimension()
   {
      return 2;
   }

   /** Computes <code>du<sub>C</sub>/dt</code> and <code>di<sub>L</sub>/dt</code>. */
   public void computeDerivatives( double t, double [] y, double [] ydot )
   {
   // TODO: calculate derivatives
      double uQ = Math.cos( omega * t );
      double uC = y[0];
      double iL = y[1];

      ydot[0] = iL / C;  // duC / dt
      ydot[1] = ( uQ - uC - R * iL ) / L;  // diL / dt
   }
}
