// import org.apache.commons.math.ode.*;
import org.apache.commons.math3.ode.*;

/** Retrieves the solution from a {@link ContinuousOutputModel}.<br>
  * <br>
  * See <a target="_blank" href="../CommonsMath_ODE_RLC_circuit~Demo.jxn">CommonsMath_ODE_RLC_circuit~Demo.jxn</a>
  * for an example.
  */  
public class JxnCOMAnalyzer
{
   private JxnCOMAnalyzer()
   {
   }

   /** Returns an array of state values <code>y<sub>iState</sub>(t<sub>k</sub>)</code> for <code> t<sub>k</sub> </code> values 
     * given in the array {@code t}.
     * @param com model available from the integrator. See {@link ODEIntegrator#addStepHandler ODEIntegrator.addStepHandler(...)}
     * and {@link ContinuousOutputModel}
     * @param t array of points in time where output is desired
     * @param iState state for which output is desired 
     * @return values of a state variable at given points in time     
     */
   public static double [] analyze( ContinuousOutputModel com, double [] t, int iState ) // throws DerivativeException
   {
      double [] yt = new double[ t.length ];
      
      for( int i = 0; i < t.length; i++ )
      {
         com.setInterpolatedTime( t[i] );
         double [] y = com.getInterpolatedState();
         yt[i] = y[iState];
      }
      
      return yt;
   }
}
