import org.apache.commons.math3.analysis.UnivariateFunction;

/** Wrapper used to pass a method definition to <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms.
  * Computes a command given as a String.<br/>
  * <br/>
  * See <a target="_blank" href="../JxnUnivariateFunction_SolverOptimizer~Demo.jxn">JxnUnivariateFunction_SolverOptimizer~Demo.jxn</a>
  * for examples.  
  */
public class JxnUnivariateFunction implements UnivariateFunction
{
   String cmd;
   KmgFormelInterpreter fi;
   static final boolean skipInputLogEnabled = true;

   /** Constructs an {@link org.apache.commons.math3.analysis.UnivariateFunction}
     * instance to be used by <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms.<br/>
     * Example:
     * <pre>
     *    uvf = @JxnUnivariateFunction( $this, "Math.sin $x;" )</pre>
     * @param fi an internal copy of {@code fi} is used to execute {@code cmd}
     * @param cmd one or more statements separated by '{@code ;}'. The command {@code cmd} must use the
     * variable double {@code $x}
     * to compute a result of type double. If {@code cmd} ends with '{@code ;}', log output of intermediate steps is suppressed.
     */
   public JxnUnivariateFunction( KmgFormelInterpreter fi, String cmd )
   {
      this.cmd = cmd;
      this.fi = new KmgFormelInterpreter( fi );
   }
   
   /** @deprecated replaced by {@link #JxnUnivariateFunction(KmgFormelInterpreter, String)} */
   public JxnUnivariateFunction( String cmd, KmgFormelInterpreter fi )
   {
      this.cmd = cmd;
      this.fi = new KmgFormelInterpreter( fi );
   }
   
   /** Internally called by CommonsMath algorithms: Computes the function value using the given argument {@code x}. */
   public double value( double x )
   {
      fi.put( "$x", new JxnPrimitiveWrapper(x) );
      return KmgFormelInterpreter.doubleValue( fi.execute( cmd, skipInputLogEnabled ) );
   }

   /** Returns the command {@code cmd} passed to the constructor. */
   public String toString()
   {
      return cmd;
   }
}
