import java.math.BigInteger;

/** Wrapper for {@code java.math.}{@link java.math.BigInteger}.<br>
  * TODO:<s>
  * Simple implementation of a fraction algebra (example of a <i>user defined algebra</i>).<br/>
  * Stores numerator and denominator with int precision. Allows mixed operations with int and double.<br/>
  * For a may be more elaborate implementation 
  * <!-- see <a href="http://commons.apache.org/proper/commons-math/apidocs/org/apache/commons/math3/fraction/Fraction.html">Commons Math</a> -->
  * see <a target="_blank" href="http://commons.apache.org/proper/commons-math/userguide/fraction.html">Commons Math</a> / 
  * <a target="_blank" href="http://commons.apache.org/proper/commons-math/apidocs/index.html?org/apache/commons/math4/fraction/BigFraction.html"><code>BigFraction</code></a>.
  * </s>
  * @see <a target="_blank" href="../../MyFractionAlgebra_UserDefinedAlgebra.html">user defined algebra</a>
  */
public class BigIntegerAlgebra extends BigInteger
{
   static final boolean DEBUG = false;
   
// private BigInteger itsBigInteger = null;
   

   /** Constructs a new BigIntegerAlgebra object. */
   public BigIntegerAlgebra( BigInteger value )
   {
   // itsBigInteger = value;
      super( value.toByteArray() );
   }

   /** Constructs a new BigIntegerAlgebra object.<br>
     * Required for mixed operations: int <i>op</i> BigInteger, BigInteger <i>op</i> int
     */
   public BigIntegerAlgebra( long value )
   {
   // itsBigInteger = BigInteger.valueOf( value );
   // super( "" + value );
      this( BigInteger.valueOf( value ) );
   }


// --- reflection methods ---

// arithmetic methods are required to return BigIntegerAlgebra 

   public BigIntegerAlgebra neg()
   {
      return new BigIntegerAlgebra( negate() );
   }
   
   /** Adds {@code opnd} to this.<br>
     * Calls {@link java.math.BigInteger#add}.
     */
   public BigIntegerAlgebra add( BigInteger opnd )
   {
      return new BigIntegerAlgebra( super.add( opnd ) );
   }
   
   public BigIntegerAlgebra sub( BigInteger opnd )
   {
      return new BigIntegerAlgebra( subtract( opnd ) );
   }
   
   /** (Post)Multiplies this by {@code opnd}.<br>
     * Calls {@link java.math.BigInteger#multiply}.
     */
   public BigIntegerAlgebra mul( BigInteger opnd )
   {
      return new BigIntegerAlgebra( multiply( opnd ) );
   }

   /** Returns this divided by {@code opnd}.<br>
     * Calls {@link java.math.BigInteger#divide}.
     */
   public BigIntegerAlgebra div( BigInteger opnd )
   {
      return new BigIntegerAlgebra( divide( opnd ) );
   }
   
   /** Returns the remainder of this divided by {@code opnd}.<br>
     * Calls {@link java.math.BigInteger#remainder}.
     */
   public BigIntegerAlgebra remainder( BigInteger opnd )
   {
      return new BigIntegerAlgebra( super.remainder( opnd ) );
   }
   
   /** Returns the power of {@code exponent} of this.<br>
     * Calls {@link java.math.BigInteger#pow}.
     */
   public BigIntegerAlgebra pow( int exponent )
   {
      return new BigIntegerAlgebra( super.pow( exponent ) );
   }


// --- other reflection methods ---

}
